-- Constants

-- URL to DocsVision
DECLARE @ServerURL AS nvarchar(128)
SET @ServerURL = N'CardID='

-- Count of days passed since last process execute time after process is treated as long ago executed
DECLARE @DaysCount AS int
SET @DaysCount = 2

-- Count of process log messages after which process is treated as process with large log
DECLARE @MaxLogMessages AS int
SET @MaxLogMessages = 10

-- Message for long ago executed process
DECLARE @LongAgoExecuted AS nvarchar(128)
SET @LongAgoExecuted = N'  '

-- Message for active processes with no active functions
DECLARE @NoActiveFunctions AS nvarchar(128)
SET @NoActiveFunctions = N'  '

-- Message for active processes with process execute time is lesser then linked card change datetime
DECLARE @ExecuteTimeLessThanLinked AS nvarchar(128)
SET @ExecuteTimeLessThanLinked = N'         '

-- Message for active processes with finished tasks
DECLARE @ProcessWithFinishedTasks AS nvarchar(128)
SET @ProcessWithFinishedTasks = N'     '

-- Message for finished processes with failed functions
DECLARE @FinishedProcessWithFailedFunctions AS nvarchar(128)
SET @FinishedProcessWithFailedFunctions = N'     '

-- Message for processes with large logs
DECLARE @ProcessWithLargeLog AS nvarchar(128)
SET @ProcessWithLargeLog = N'   '


-- Table for instance IDs
DECLARE @Instances TABLE (InstanceID uniqueidentifier, [Desc] nvarchar(128))


-- Now
DECLARE @Now AS datetime
SET @Now = GETDATE()

DECLARE @OneHourAgo AS datetime
SET @OneHourAgo = DATEADD(hh, -1, @Now)


-- Selecting process IDs which last execute time is beyond limits
INSERT INTO @Instances
SELECT DISTINCT tMain.[InstanceID], @LongAgoExecuted FROM [dvtable_{0EF6BCCA-7A09-4027-A3A2-D2EEECA1BF4D}] AS tMain
JOIN [dvsys_instances] AS tInst  ON tMain.[InstanceID] = tInst.[InstanceID]
WHERE (ISNULL(tInst.[Deleted], 0) = 0) AND ((ISNULL(tInst.[Template], 0) = 0) AND (tMain.[State] = 1))
 AND LastRunDate <= DATEADD(d, -@DaysCount, @Now)


-- Selecting process IDs which are active with no really active functions
INSERT INTO @Instances
SELECT DISTINCT tMain.[InstanceID], @NoActiveFunctions FROM [dvtable_{0EF6BCCA-7A09-4027-A3A2-D2EEECA1BF4D}] AS tMain
JOIN [dvsys_instances] AS tInst  ON tMain.[InstanceID] = tInst.[InstanceID]
LEFT JOIN [dbo].[dvtable_{97CC73BA-1953-4A70-8460-415BD4BCAAAE}] tpassstates WITH(NOLOCK) ON (tpassstates.[InstanceID] = tMain.[InstanceID])
AND
(
  -- Wait
  (tpassstates.[State] = 1)

  -- Active
  OR (tpassstates.[State] = 2)

  -- Done
  OR (tpassstates.[State] = 3)

  --  Error
  OR (tpassstates.[State] = 5)

  -- Waiting for changes in process
  OR (tpassstates.[State] = 6)

  -- Waiting for changes in linked card
  OR (tpassstates.[State] = 7)

  -- Waiting for changes in weakly-linked card
  OR (tpassstates.[State] = 8)
)
WHERE (ISNULL(tInst.[Deleted], 0) = 0) AND ((ISNULL(tInst.[Template], 0) = 0) AND (tMain.[State] = 1))
GROUP BY tMain.[InstanceID]
HAVING COUNT(tpassstates.[State]) = 0

-- Selecting process IDs which execute time is lesser than linked card change time
INSERT INTO @Instances
SELECT DISTINCT tMain.[InstanceID], @ExecuteTimeLessThanLinked FROM [dvtable_{0EF6BCCA-7A09-4027-A3A2-D2EEECA1BF4D}] AS tMain
JOIN [dvsys_instances] AS tInst  ON tMain.[InstanceID] = tInst.[InstanceID]
LEFT JOIN [dbo].[dvtable_{97CC73BA-1953-4A70-8460-415BD4BCAAAE}] tpassstates WITH(NOLOCK) ON (tpassstates.[InstanceID] = tMain.[InstanceID])
LEFT JOIN [dbo].[dvtable_{10105DC1-8B61-4A76-B719-02D679662606}] tfunctions WITH(NOLOCK) ON (tpassstates.[ParentRowID] = tfunctions.[RowID])
LEFT JOIN [dbo].[dvsys_instances_date] tInst1 WITH(NOLOCK) ON tfunctions.[CardID] = tInst1.[InstanceID]
LEFT JOIN [dbo].[dvsys_instances_date] tInst2 WITH(NOLOCK) ON tfunctions.[WeakCardID] = tInst2.[InstanceID]
WHERE (ISNULL(tInst.[Deleted], 0) = 0) AND ((ISNULL(tInst.[Template], 0) = 0) AND (tMain.[State] = 1))
AND
(
  -- Waiting for changes in linked card
  ((tpassstates.[State] = 7) AND (tfunctions.[CardID] IS NULL OR (tInst1.[ChangeDateTime] >= tMain.[LastRunDate] AND tInst1.[ChangeDateTime] <= @OneHourAgo)))

  -- Waiting for changes in weakly-linked card
  OR ((tpassstates.[State] = 8) AND (tfunctions.[WeakCardID] IS NULL OR (tInst2.[ChangeDateTime] >= tMain.[LastRunDate] AND tInst2.[ChangeDateTime] <= @OneHourAgo)))
)


-- Selecting process IDs with active Task function with attached finished task
INSERT INTO @Instances
SELECT DISTINCT tMain.[InstanceID], @ProcessWithFinishedTasks FROM [dvtable_{0EF6BCCA-7A09-4027-A3A2-D2EEECA1BF4D}] AS tMain
JOIN [dvsys_instances] AS tInst  ON tMain.[InstanceID] = tInst.[InstanceID]
LEFT JOIN [dbo].[dvtable_{97CC73BA-1953-4A70-8460-415BD4BCAAAE}] tpassstates WITH(NOLOCK) ON (tpassstates.[InstanceID] = tMain.[InstanceID])
LEFT JOIN [dbo].[dvtable_{10105DC1-8B61-4A76-B719-02D679662606}] tfunctions WITH(NOLOCK) ON (tpassstates.[ParentRowID] = tfunctions.[RowID])
LEFT JOIN [dbo].[dvsys_instances_date] tInst1 WITH(NOLOCK) ON tfunctions.[CardID] = tInst1.[InstanceID]
LEFT JOIN [dbo].[dvtable_{D48E6155-C774-4205-AB70-7A67AB69DF22}] tTaskPerforming WITH(NOLOCK) ON tfunctions.[CardID] = tTaskPerforming.[InstanceID]
WHERE (ISNULL(tInst.[Deleted], 0) = 0) AND ((ISNULL(tInst.[Template], 0) = 0) AND (tMain.[State] = 1))
AND (tpassstates.[State] = 7) AND ((tTaskPerforming.[TaskState] = 5) OR(tTaskPerforming.[TaskState] = 6) OR (tTaskPerforming.[TaskState] = 7) )
AND tInst1.[ChangeDateTime] < @OneHourAgo


-- Selecting process IDs which are finished with failed functions
INSERT INTO @Instances
SELECT DISTINCT tMain.[InstanceID], @FinishedProcessWithFailedFunctions FROM [dvtable_{0EF6BCCA-7A09-4027-A3A2-D2EEECA1BF4D}] AS tMain
JOIN [dvsys_instances] AS tInst  ON tMain.[InstanceID] = tInst.[InstanceID]
LEFT JOIN [dbo].[dvtable_{97CC73BA-1953-4A70-8460-415BD4BCAAAE}] tpassstates WITH(NOLOCK) ON (tpassstates.[InstanceID] = tMain.[InstanceID])
WHERE (ISNULL(tInst.[Deleted], 0) = 0) AND ((ISNULL(tInst.[Template], 0) = 0) AND (tMain.[State] = 4))
AND (tpassstates.[State] = 5)


-- Selecting process IDs with large logs
INSERT INTO @Instances
SELECT DISTINCT tMain.[InstanceID], @ProcessWithLargeLog FROM [dvtable_{0EF6BCCA-7A09-4027-A3A2-D2EEECA1BF4D}] AS tMain
JOIN [dvsys_instances] AS tInst  ON tMain.[InstanceID] = tInst.[InstanceID]
LEFT JOIN [dbo].[dvtable_{388F390F-139E-498E-A461-A24FBA160487}] tlog WITH(NOLOCK) ON (tlog.[InstanceID] = tMain.[InstanceID])
WHERE (ISNULL(tInst.[Deleted], 0) = 0) --AND ((ISNULL(tInst.[Template], 0) = 0) AND (tMain.[State] = 1))
GROUP BY tMain.[InstanceID]
HAVING COUNT(*) > @MaxLogMessages


SELECT t0.[Desc] AS 'Possible problem' , tInst.[Description], (@ServerURL + CONVERT(nvarchar(36), t0.[InstanceID])) AS 'Process url'
FROM @Instances t0
JOIN [dbo].[dvsys_instances] tInst WITH(NOLOCK) ON tInst.[InstanceID] = t0.[InstanceID]
